import React from 'react';
import ReactDOM from 'react-dom/client';
//import './index.css';
import store from './store';
import { Provider } from 'react-redux';
import App from './App';
import KeyEvents from './components/KeyEvents'

// Boolean for determining if the liveblog has been initialized
let initialized = false;

const initializeLiveblog = (event) => {

  // Bail if it's already initialized
  if (initialized) {
    return;
  }

  requestAnimationFrame(() => {

    // Render the main liveblog app
    const rootEl = document.getElementById('dmg-liveblog-root');
    const preloadedHTML = rootEl.innerHTML;
    const liveblogRoot = ReactDOM.createRoot(rootEl);
    liveblogRoot.render(
      <Provider store={store}>
        <App preloadedHTML={rootEl.innerHTML} />
      </Provider>
    );

    // Render the key events elsewhere on the page we've specified in the config
    if (window.DMGLiveblog?.keyEvents?.active && window.DMGLiveblog.keyEvents?.container) {

      // Prepare to create a root container
      let keyEventsContainer = false;
      const keyEvents = window.DMGLiveblog.keyEvents;
      const keyEventsRootID = 'dmg-liveblog-key-events';

      if (keyEvents?.container?.selector && document.querySelector(keyEvents.container.selector)) {

        // Render within a specific selector
        keyEventsContainer = document.querySelector(keyEvents.container.selector);
      } else {

        // Generate the root container
        let keyEventContainer = document.createElement('div');
        keyEventContainer.setAttribute('id', keyEventsRootID);

        if (keyEvents?.container?.appendTo && document.querySelector(keyEvents.container.appendTo)) {

          // Append the root to another container
          document.querySelector(keyEvents.container.appendTo).append(keyEventContainer)
        } else if (keyEvents?.container?.insertBefore && document.querySelector(keyEvents.container.insertBefore)) {

          // Insert the root before another element
          const insertBefore = document.querySelector(keyEvents.container.insertBefore);
          insertBefore.parentNode.insertBefore(keyEventContainer, insertBefore)
        } else if (keyEvents?.container?.insertAfter && document.querySelector(keyEvents.container.insertAfter)) {

          // Insert the root after another element
          const insertAfter = document.querySelector(keyEvents.container.insertAfter);
          insertAfter.parentNode.insertBefore(keyEventContainer, insertAfter.nextSibling)
        }

        keyEventsContainer = document.getElementById(keyEventsRootID)
      }

      // If we have a root, render the React component
      if (keyEventsContainer) {
        const keyEventsRoot = ReactDOM.createRoot(keyEventsContainer);
        keyEventsRoot.render(
          <Provider store={store}>
            <KeyEvents />
          </Provider>
        );
      }
    }
  });

  // Set flag
  initialized = true;
};

// Is initialization deferred?
if (window?.DMGLiveblog?.deferInitialization) {

  // Initiate after first user scroll or click
  document.addEventListener('scroll', initializeLiveblog, {once: true});
  document.addEventListener('click', initializeLiveblog, {once: true});
} else {

  // Initialize immediately
  initializeLiveblog();
}
